package fr.nitorac.aurionweb;

import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.NavigationUI;

import com.google.android.material.bottomnavigation.BottomNavigationView;

import java.io.IOException;
import java.security.GeneralSecurityException;

import fr.nitorac.aurionweb.aurionweb.AurionCred;
import fr.nitorac.aurionweb.aurionweb.AurionManager;
import fr.nitorac.aurionweb.security.SecurityManager;
import fr.nitorac.aurionweb.update.UpdateManager;

public class MainActivity extends AppCompatActivity {

    private static MainActivity mainActivity;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        mainActivity = this;
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        UpdateManager.getInstance().checkUpdates(this);

        try{
            SecurityManager.loadSavedCreds(this);
        }catch (GeneralSecurityException | IOException e){
            e.printStackTrace();
            Toast.makeText(this, "Impossible de récupérer les identifiants sauvegardés :(", Toast.LENGTH_LONG).show();
            SecurityManager.creds = new AurionCred();
        }

        BottomNavigationView navView = findViewById(R.id.nav_view);
        NavController navController = Navigation.findNavController(this, R.id.nav_host_fragment);
        NavigationUI.setupWithNavController(navView, navController);
        navView.setOnNavigationItemReselectedListener((ignored) -> {});
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.appbar_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.connect_menuitem){
            try{
                AurionManager.getInstance().spawnLoginDialog(this);
            }catch (Exception e){
                Toast.makeText(this, "Impossible d'afficher la connexion (Erreur : " + e.getLocalizedMessage() + ")", Toast.LENGTH_LONG).show();
            }
            return true;
        }
        return super.onOptionsItemSelected(item);
    }


    @Override
    protected void onPause() {
        super.onPause();
        UpdateManager.handleOnPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        UpdateManager.handleOnResume();
    }

    public static MainActivity getInstance(){
        return mainActivity;
    }
}