package fr.nitorac.aurionweb.security;

import android.os.Build;
import android.security.keystore.KeyGenParameterSpec;
import android.security.keystore.KeyProperties;
import android.util.Base64;

import androidx.annotation.RequiresApi;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.security.KeyStore;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.GCMParameterSpec;

@RequiresApi(api = Build.VERSION_CODES.M)
public class SecurityHelperPostM implements SecurityHelper {

    private static final String AndroidKeyStore = "AndroidKeyStore";
    private static final String AES_MODE = "AES/GCM/NoPadding";

    private KeyStore keyStore;

    public SecurityHelperPostM() throws GeneralSecurityException, IOException {
        init();
    }

    public void init() throws GeneralSecurityException, IOException {
        keyStore = KeyStore.getInstance(AndroidKeyStore);
        keyStore.load(null);

        if (!keyStore.containsAlias(SecurityManager.KEYSTORE_NAME)) {
            KeyGenerator keyGenerator = KeyGenerator.getInstance(KeyProperties.KEY_ALGORITHM_AES, AndroidKeyStore);
            keyGenerator.init(
                    new KeyGenParameterSpec.Builder(SecurityManager.KEYSTORE_NAME,
                            KeyProperties.PURPOSE_ENCRYPT | KeyProperties.PURPOSE_DECRYPT)
                            .setBlockModes(KeyProperties.BLOCK_MODE_GCM)
                            .setEncryptionPaddings(KeyProperties.ENCRYPTION_PADDING_NONE)
                            .setRandomizedEncryptionRequired(true)
                            .build());
            keyGenerator.generateKey();
        }
    }

    private Key getSecretKey() throws GeneralSecurityException {
        return keyStore.getKey(SecurityManager.KEYSTORE_NAME, null);
    }

    public String encrypt(String raw) throws GeneralSecurityException {
        Cipher c = Cipher.getInstance(AES_MODE);
        c.init(Cipher.ENCRYPT_MODE, getSecretKey(), c.getParameters());
        byte[] encodedBytes = c.doFinal(raw.getBytes(StandardCharsets.UTF_8));
        return Base64.encodeToString(encodedBytes, Base64.DEFAULT).replace("\n", "") + ":" + Base64.encodeToString(c.getIV(), Base64.DEFAULT);
    }

    public String decrypt(String encrypted) throws GeneralSecurityException  {
        String[] data = encrypted.split(":");
        Cipher c = Cipher.getInstance(AES_MODE);
        c.init(Cipher.DECRYPT_MODE, getSecretKey(), new GCMParameterSpec(128, Base64.decode(data[1], Base64.DEFAULT)));
        return new String(c.doFinal(Base64.decode(data[0], Base64.DEFAULT)), StandardCharsets.UTF_8);
    }
}